@@ -5,28 +5,38 @@ from joblib import Parallel, delayed
 from tensorpac.config import CONFIG
 
 
-def compute_surrogates(pha, amp, ids, fcn, n_perm, n_jobs):
+def compute_surrogates(pha, amp, ids, fcn, n_perm, n_jobs, random_state):
     """Compute surrogates using tensors and parallel computing."""
     if ids == 0:
         return None
     else:
         fcn_p = {1: swap_pha_amp, 2: swap_blocks, 3: time_lag}[ids]
 
-    def para_surr():  # noqa
-        return fcn(*fcn_p(pha, amp))
+    def para_surr(rnd):  # noqa
+        return fcn(*fcn_p(pha, amp, random_state=random_state + rnd))
     s = Parallel(n_jobs=n_jobs, **CONFIG['JOBLIB_CFG'])(delayed(para_surr)(
-    ) for k in range(n_perm))
+        k) for k in range(n_perm))
     return np.array(s)
 
 
-def swap_pha_amp(pha, amp):
-    """Swap phase / amplitude trials.
+def swap_pha_amp(pha, amp, random_state=None):
+    """Compute surrogates by swapping phase / amplitude trials.
+
+    This function destroys the relation between the phase and the amplitude in
+    order to correct for PAC that could be obtained by chance. To this end,
+    this function reorganize the relation from trial-to-trial between the phase
+    and the amplitude (swapping). In particular, in that case it is the trials
+    of the phase that are randomly swapped and the amplitude is leaved
+    unchanged. For a more detailed description, see :cite:`tort2010measuring`
 
     Parameters
     ----------
     pha, amp : array_like
-        Respectively the arrays of phases of shape (n_pha, ..., n_times) and
-        the array of amplitudes of shape (n_amp, ..., n_times).
+        Respectively the arrays of phases of shape
+        (n_pha, n_trials, ..., n_times) and the array of amplitudes of shape
+        (n_amp, n_trials, ..., n_times).
+    random_state : int | None
+        Fix the random state of the machine for reproducible results.
 
     Returns
     -------
@@ -36,22 +46,31 @@ def swap_pha_amp(pha, amp):
 
     References
     ----------
-    Tort ABL, Komorowski R, Eichenbaum H, Kopell N (2010) Measuring
-    Phase-Amplitude Coupling Between Neuronal Oscillations of Different
-    Frequencies. Journal of Neurophysiology 104:1195–1210.
+    Tort et al. 2010 :cite:`tort2010measuring`
     """
-    tr_ = np.random.permutation(pha.shape[1])
+    if random_state is None:
+        random_state = int(np.random.randint(0, 10000, size=1))
+    rnd = np.random.RandomState(random_state)
+    tr_ = rnd.permutation(pha.shape[1])
     return pha[:, tr_, ...], amp
 
 
-def swap_blocks(pha, amp):
-    """Swap amplitudes time blocks.
+def swap_blocks(pha, amp, random_state=None):
+    """Compute surrogates by swapping amplitudes time blocks.
+
+    This function destroys the relation between the phase and the amplitude in
+    order to correct for PAC that could be obtained by chance. To this end,
+    this function cut the amplitude at a random time point. Then, both time
+    blocks are swapped. For a more detailed description, see
+    :cite:`bahramisharif2013propagating`.
 
     Parameters
     ----------
     pha, amp : array_like
         Respectively the arrays of phases of shape (n_pha, ..., n_times) and
         the array of amplitudes of shape (n_amp, ..., n_times).
+    random_state : int | None
+        Fix the random state of the machine for reproducible results.
 
     Returns
     -------
@@ -61,13 +80,13 @@ def swap_blocks(pha, amp):
 
     References
     ----------
-    Bahramisharif A, van Gerven MAJ, Aarnoutse EJ, Mercier MR, Schwartz TH,
-    Foxe JJ, Ramsey NF, Jensen O (2013) Propagating Neocortical Gamma Bursts
-    Are Coordinated by Traveling Alpha Waves. Journal of Neuroscience
-    33:18849–18854.
+    Bahramisharif et al. 2013 :cite:`bahramisharif2013propagating`
     """
+    if random_state is None:
+        random_state = int(np.random.randint(0, 10000, size=1))
+    rnd = np.random.RandomState(random_state)
     # random cutting point along time axis
-    cut_at = np.random.randint(1, amp.shape[-1], (1,))
+    cut_at = rnd.randint(1, amp.shape[-1], (1,))
     # Split amplitude across time into two parts :
     ampl = np.array_split(amp, cut_at, axis=-1)
     # Revered elements :
@@ -75,14 +94,21 @@ def swap_blocks(pha, amp):
     return pha, np.concatenate(ampl, axis=-1)
 
 
-def time_lag(pha, amp):
-    """Introduce a time lag on phase series.
+def time_lag(pha, amp, random_state=None):
+    """Compute surrogates by introducing a time lag on phase series.
+
+    This function destroys the relation between the phase and the amplitude in
+    order to correct for PAC that could be obtained by chance. To this end, a
+    random temporal lag is introduced at the beginning of the phase. For a more
+    detailed description, see :cite:`canolty2006high`.
 
     Parameters
     ----------
     pha, amp : array_like
         Respectively the arrays of phases of shape (n_pha, ..., n_times) and
         the array of amplitudes of shape (n_amp, ..., n_times).
+    random_state : int | None
+        Fix the random state of the machine for reproducible results.
 
     Returns
     -------
@@ -92,10 +118,12 @@ def time_lag(pha, amp):
 
     References
     ----------
-    Canolty RT (2006) High Gamma Power Is Phase-Locked to Theta. science
-    1128115:313.
+    Canolty et al. 2006 :cite:`canolty2006high`
     """
-    shift = np.random.randint(pha.shape[-1])
+    if random_state is None:
+        random_state = int(np.random.randint(0, 10000, size=1))
+    rnd = np.random.RandomState(random_state)
+    shift = rnd.randint(pha.shape[-1])
     return np.roll(pha, shift, axis=-1), amp
 
 
