@@ -134,6 +134,44 @@ class _PacObj(object):
         pha = (pha + np.pi) % (2. * np.pi) - np.pi
         return pha, amp
 
+    def _infer_pvalues(self, effect, perm, p=.05, mcp='maxstat'):
+        """Global function for statistical inferences.
+
+        In order to work this method requires :
+
+            * effect = array like of shape (n_dims...)
+            * perm = array like of shape (n_perm, n_dims...)
+        """
+        assert all([isinstance(k, np.ndarray) for k in (effect, perm)])
+        n_perm = perm.shape[0]
+        # compute the minimum number of required permutations
+        n_perm_req = int(10. / p)
+        if n_perm < n_perm_req:
+            logger.warning(f"For inferences at p<{p}, it is recommended to per"
+                           f"form at least n_perm={n_perm_req} permutations")
+
+        # ---------------------------------------------------------------------
+        logger.info(f"    infer p-values at (p={p}, mcp={mcp})")
+        # computes the pvalues
+        if mcp is 'maxstat':
+            max_p = perm.reshape(n_perm, -1).max(1)[np.newaxis, ...]
+            nb_over = (effect[..., np.newaxis] <= max_p).sum(-1)
+            pvalues = nb_over / n_perm
+            # non-signi. p-values are set to 1. and min(pvalues) = 1 / n_perm
+            pvalues[pvalues >= p] = 1.
+            pvalues = np.maximum(1. / n_perm, pvalues)
+        elif mcp in ['fdr', 'bonferroni']:
+            from mne.stats import fdr_correction, bonferroni_correction
+            fcn = fdr_correction if mcp is 'fdr' else bonferroni_correction
+            # compute the p-values
+            pvalues = (effect[np.newaxis, ...] <= perm).sum(0) / n_perm
+            pvalues = np.maximum(1. / n_perm, pvalues)
+            # apply correction
+            is_signi, pvalues = fcn(pvalues, alpha=p)
+            pvalues[~is_signi] = 1.
+
+        return pvalues
+
     @property
     def f_pha(self):
         """Vector of phases of shape (n_pha, 2)."""
@@ -205,27 +243,39 @@ class Pac(_PacObj, _PacPlt):
 
         * First digit : refer to the pac method
 
-            - '1' : Mean Vector Length (MVL) [#f1]_
-            - '2' : Kullback-Leibler Distance (KLD) [#f2]_
-            - '3' : Heights Ratio (HR) [#f3]_
-            - '4' : ndPAC [#f4]_
-            - '5' : Phase Synchrony [#f3]_
-            - '6' : Gaussian Copula PAC [#f7]_
+            - 1 : Mean Vector Length (MVL) :cite:`canolty2006high`
+              (see :func:`tensorpac.methods.mean_vector_length`)
+            - 2 : Modulation Index (MI) :cite:`tort2010measuring`
+              (see :func:`tensorpac.methods.modulation_index`)
+            - 3 : Heights Ratio (HR) :cite:`lakatos2005oscillatory`
+              (see :func:`tensorpac.methods.heights_ratio`)
+            - 4 : ndPAC :cite:`ozkurt2012statistically`
+              (see :func:`tensorpac.methods.norm_direct_pac`)
+            - 5 : Phase-Locking Value (PLV)
+              :cite:`penny2008testing,lachaux1999measuring`
+              (see :func:`tensorpac.methods.phase_locking_value`)
+            - 6 : Gaussian Copula PAC (GCPAC) :cite:`ince2017statistical`
+              (see :func:`tensorpac.methods.gauss_cop_pac`)
 
         * Second digit : refer to the method for computing surrogates
 
-            - '0' : No surrogates
-            - '1' : Swap phase / amplitude across trials [#f2]_
-            - '2' : Swap amplitude time blocks [#f5]_
-            - '3' : Time lag [#f1]_
+            - 0 : No surrogates
+            - 1 : Swap phase / amplitude across trials
+              :cite:`tort2010measuring`
+              (see :func:`tensorpac.methods.swap_pha_amp`)
+            - 2 : Swap amplitude time blocks
+              :cite:`bahramisharif2013propagating`
+              (see :func:`tensorpac.methods.swap_blocks`)
+            - 3 : Time lag :cite:`canolty2006high`
+              (see :func:`tensorpac.methods.time_lag`)
 
         * Third digit : refer to the normalization method for correction
 
-            - '0' : No normalization
-            - '1' : Substract the mean of surrogates
-            - '2' : Divide by the mean of surrogates
-            - '3' : Substract then divide by the mean of surrogates
-            - '4' : Z-score
+            - 0 : No normalization
+            - 1 : Substract the mean of surrogates
+            - 2 : Divide by the mean of surrogates
+            - 3 : Substract then divide by the mean of surrogates
+            - 4 : Z-score
 
     f_pha, f_amp : list/tuple/array | def: [2, 4] and [60, 200]
         Frequency vector for the phase and amplitude. Here you can use
@@ -236,7 +286,10 @@ class Pac(_PacObj, _PacPlt):
             * Dynamic definition : (start, stop, width, step)
             * Range definition (ex : np.arange(3) => [[0, 1], [1, 2]])
             * Using a string. `f_pha` and `f_amp` can be 'lres', 'mres', 'hres'
-              respectively for low, middle and high resolution vectors
+              respectively for low, middle and high resolution vectors. In that
+              case, it uses the definition proposed by Bahramisharif et al.
+              2013 :cite:`bahramisharif2013propagating` i.e
+              f_pha = [f - f / 4, f + f / 4] and f_amp = [f - f / 8, f + f / 8]
 
     dcomplex : {'wavelet', 'hilbert'}
         Method for the complex definition. Use either 'hilbert' or
@@ -245,26 +298,12 @@ class Pac(_PacObj, _PacPlt):
         Control the number of cycles for filtering (only if dcomplex is
         'hilbert'). Should be a tuple of integers where the first one
         refers to the number of cycles for the phase and the second for the
-        amplitude [#f5]_.
+        amplitude :cite:`bahramisharif2013propagating`.
     width : int | 7
         Width of the Morlet's wavelet.
     n_bins : int | 18
-        Number of bins for the KLD and HR PAC method [#f2]_ [#f3]_
-
-    References
-    ----------
-    .. [#f1] `Canolty et al, 2006 <http://www.ncbi.nlm.nih.gov/pmc/articles/
-       PMC2628289/>`_
-    .. [#f2] `Tort et al, 2010 <http://www.ncbi.nlm.nih.gov/pmc/articles/
-       PMC2941206/>`_
-    .. [#f3] `Lakatos et al, 2005 <https://www.ncbi.nlm.nih.gov/pubmed/
-       15901760>`_
-    .. [#f4] `Ozkurt et al, 2012 <http://www.ncbi.nlm.nih.gov/pubmed/
-       22531738/>`_
-    .. [#f5] `Bahramisharif et al, 2013 <http://www.jneurosci.org/content/33/
-       48/18849.short/>`_
-    .. [#f7] `Ince et al, 2017 <https://onlinelibrary.wiley.com/doi/full/10.
-       1002/hbm.23471>`_
+        Number of bins for the KLD and HR PAC method
+        :cite:`tort2010measuring,lakatos2005oscillatory`
     """
 
     def __init__(self, idpac=(1, 2, 3), f_pha=[2, 4], f_amp=[60, 200],
@@ -279,7 +318,8 @@ class Pac(_PacObj, _PacPlt):
         self.n_bins = int(n_bins)
         logger.info("Phase Amplitude Coupling object defined")
 
-    def fit(self, pha, amp, n_perm=200, p=.05, n_jobs=-1, verbose=None):
+    def fit(self, pha, amp, n_perm=200, p=.05, mcp='maxstat', n_jobs=-1,
+            random_state=None, verbose=None):
         """Compute PAC on filtered data.
 
         Parameters
@@ -293,9 +333,17 @@ class Pac(_PacObj, _PacPlt):
             Number of surrogates to compute.
         p : float | 0.05
             Statistical threshold
+        mcp : {'fdr', 'bonferroni'}
+            Correct the p-values for multiple comparisons. Use either :
+
+                * 'maxstat' : maximum statistics
+                * 'fdr' : FDR correction (need MNE-Python)
+                * 'bonferroni' : Bonferroni correction (need MNE-Python)
         n_jobs : int | -1
             Number of jobs to compute PAC in parallel. For very large data,
             set this parameter to 1 in order to prevent large memory usage.
+        random_state : int | None
+            Fix the random state of the machine for reproducible results.
 
         Returns
         -------
@@ -317,7 +365,7 @@ class Pac(_PacObj, _PacPlt):
         # input checking
         pha, amp = self._phampcheck(pha, amp)
         self._pvalues, self._surrogates = None, None
-        # for the phase synchrony, extract the phase of the amplitude
+        # for the plv, extract the phase of the amplitude
         if self._idpac[0] == 5:
             amp = np.angle(hilbertm(amp))
 
@@ -348,14 +396,16 @@ class Pac(_PacObj, _PacPlt):
         # ---------------------------------------------------------------------
         # compute surrogates (if needed)
         if compute_surro:
+            if random_state is None:
+                random_state = int(np.random.randint(0, 10000, size=1))
             logger.info(f"    compute surrogates ({self.str_surro}, {n_perm} "
-                        "permutations)")
+                        f"permutations, random_state={random_state})")
             surro = compute_surrogates(pha, amp, self.idpac[1], fcn, n_perm,
-                                       n_jobs)
+                                       n_jobs, random_state)
             self._surrogates = surro
 
             # infer pvalues
-            self.infer_pvalues(p)
+            self.infer_pvalues(p, mcp=mcp)
 
         # ---------------------------------------------------------------------
         # normalize (if needed)
@@ -367,8 +417,9 @@ class Pac(_PacObj, _PacPlt):
 
         return pac
 
-    def filterfit(self, sf, x_pha, x_amp=None, n_perm=200, p=.05, n_jobs=-1,
-                  edges=None, verbose=None):
+    def filterfit(self, sf, x_pha, x_amp=None, n_perm=200, p=.05,
+                  mcp='maxstat', edges=None, n_jobs=-1, random_state=None,
+                  verbose=None):
         """Filt the data then compute PAC on it.
 
         Parameters
@@ -385,11 +436,19 @@ class Pac(_PacObj, _PacPlt):
             Number of surrogates to compute.
         p : float | 0.05
             Statistical threshold
+        mcp : {'fdr', 'bonferroni'}
+            Correct the p-values for multiple comparisons. Use either :
+
+                * 'maxstat' : maximum statistics
+                * 'fdr' : FDR correction (need MNE-Python)
+                * 'bonferroni' : Bonferroni correction (need MNE-Python)
         edges : int | None
             Number of samples to discard to avoid edge effects due to filtering
         n_jobs : int | -1
             Number of jobs to compute PAC in parallel. For very large data,
             set this parameter to 1 in order to prevent large memory usage.
+        random_state : int | None
+            Fix the random state of the machine for reproducible results.
 
         Returns
         -------
@@ -413,7 +472,7 @@ class Pac(_PacObj, _PacPlt):
         assert x_pha.shape == x_amp.shape, ("Inputs `x_pha` and `x_amp` must "
                                             "have the same shape.")
         # Extract phase (npha, ...) and amplitude (namp, ...) :
-        logger.info(f"    Extract phases (n_pha={len(self.xvec)}) and "
+        logger.info(f"    extract phases (n_pha={len(self.xvec)}) and "
                     f"amplitudes (n_amps={len(self.yvec)})")
         kw = dict(keepfilt=False, edges=edges, n_jobs=1)
         pha = self.filter(sf, x_pha, 'phase', **kw)
@@ -424,42 +483,39 @@ class Pac(_PacObj, _PacPlt):
             amp = np.angle(hilbertm(amp))
 
         # Compute pac :
-        return self.fit(pha, amp, p=p, n_perm=n_perm, n_jobs=n_jobs,
-                        verbose=verbose)
+        return self.fit(pha, amp, p=p, mcp=mcp, n_perm=n_perm, n_jobs=n_jobs,
+                        random_state=random_state, verbose=verbose)
 
-    def infer_pvalues(self, p=0.05):
+    def infer_pvalues(self, p=0.05, mcp='maxstat'):
         """Infer p-values based on surrogate distribution.
 
         Parameters
         ----------
         p : float | 0.05
-            Statistical threshold
+            Significiency threshold
 
         Returns
         -------
         pvalues : array_like
             Array of p-values of shape (n_amp, n_pha)
+        mcp : {'fdr', 'bonferroni'}
+            Correct the p-values for multiple comparisons. Use either :
+
+                * 'maxstat' : maximum statistics
+                * 'fdr' : FDR correction (need MNE-Python)
+                * 'bonferroni' : Bonferroni correction (need MNE-Python)
         """
         # ---------------------------------------------------------------------
         # check that pac and surrogates has already been computed
         assert hasattr(self, 'pac'), ("You should compute PAC first. Use the "
                                       "`fit` method")
         assert hasattr(self, 'surrogates'), "No surrogates computed"
-        assert all([isinstance(k, np.ndarray) for k in (
-            self.pac, self.surrogates)])
-        n_perm = self.surrogates.shape[0]
 
-        # ---------------------------------------------------------------------
         # mean pac and surrogates across trials
         m_pac, m_surro = self.pac.mean(2), self.surrogates.mean(3)
-        self._pvalues = np.ones_like(m_pac)
-        # infer pvalues
-        logger.info(f"    Infer p-values at p={p}")
-        max_dist = m_surro.reshape(n_perm, -1).max(1)
-        th = np.percentile(max_dist, 100. * (1 - p), axis=0,
-                           interpolation='nearest')
-        self._pvalues[m_pac > th] = p
-        return self.pvalues
+        self._pvalues = self._infer_pvalues(m_pac, m_surro, p=p, mcp=mcp)
+
+        return self._pvalues
 
     def _idcheck(self, idpac):
         """Check the idpac parameter."""
@@ -469,6 +525,12 @@ class Pac(_PacObj, _PacPlt):
         # Ozkurt PAC case (doesn't need surrogates and normalization)
         if idpac[0] == 4:
             idpac = np.array([4, 0, 0])
+        if (idpac[0] == 1) and (idpac[1] == 0) and (idpac[2] == 0):
+            logger.warning(
+                "MVL is amplitude dependent which means that if the amplitude "
+                "increases, MVL also increases. You should select a "
+                "normalization method for correcting this limitation "
+                "(e.g idpac=(1, 2, 4))")
         if (idpac[2] != 0) and (idpac[1] == 0):
             logger.warning("If you want to normalize the estimated PAC, you "
                            "should select a surrogate method (second digit of "
@@ -544,11 +606,11 @@ class EventRelatedPac(_PacObj, _PacVisual):
         logger.info("Event Related PAC object defined")
 
     def fit(self, pha, amp, method='circular', smooth=None, n_jobs=-1,
-            n_perm=None, p=.05, verbose=None):
+            n_perm=None, p=.05, mcp='fdr', verbose=None):
         """Compute the Event-Related Phase-Amplitude Coupling (ERPAC).
 
-        The ERPAC [#f6]_ is used to measure PAC across trials and is
-        interesting for real-time estimation.
+        The ERPAC :cite:`voytek2013method` is used to measure PAC across trials
+        and is interesting for real-time estimation.
 
         Parameters
         ----------
@@ -558,7 +620,8 @@ class EventRelatedPac(_PacObj, _PacVisual):
             oscillations of shape (n_pha, n_epochs, n_times).
         method : {'circular', 'gc'}
             Name of the method for computing erpac. Use 'circular' for
-            reproducing [#f6]_ or 'gc' for a Gaussian-Copula based erpac.
+            reproducing :cite:`voytek2013method` or 'gc' for a Gaussian-Copula
+            based erpac :cite:`ince2017statistical`.
         smooth : int | None
             Half number of time-points to use to produce a smoothing. Only
             active with the Gaussian-Copula ('gc') method.
@@ -568,18 +631,15 @@ class EventRelatedPac(_PacObj, _PacVisual):
             swapping phase trials
         p : float | 0.05
             Statistical threshold for the gaussian-copula ('gc') method
+        mcp : {'fdr', 'bonferroni'}
+            Correct the p-values for multiple comparisons. This is needed when
+            using the circular ERPAC (:cite:`voytek2013method`). Note that the
+            correction is performed using MNE-Python.
 
         Returns
         -------
         erpac : array_like
             The ERPAC estimation of shape (n_amp, n_pha, n_times)
-
-        References
-        ----------
-        .. [#f6] `Voytek et al, 2013 <https://www.ncbi.nlm.nih.gov/pubmed/
-           22986076>`_
-        .. [#f7] `Ince et al, 2017 <https://onlinelibrary.wiley.com/doi/full/10
-           .1002/hbm.23471>`_
         """
         set_log_level(verbose)
         pha, amp = self._phampcheck(pha, amp)
@@ -592,8 +652,9 @@ class EventRelatedPac(_PacObj, _PacVisual):
             self.method = "ERPAC (Voytek et al. 2013)"
             logger.info(f"    Compute {self.method}")
             self._erpac, self._pvalues = erpac(pha, amp)
+            self.infer_pvalues(p=p, mcp=mcp)
         elif method == 'gc':
-            self.method = "Gaussian-Copula ERPAC (Ince et al. 2017)"
+            self.method = "Gaussian-Copula ERPAC"
             logger.info(f"    Compute {self.method}")
             # copnorm phases and amplitudes then compute erpac
             sco = copnorm(np.stack([np.sin(pha), np.cos(pha)], axis=-2))
@@ -604,11 +665,12 @@ class EventRelatedPac(_PacObj, _PacVisual):
                 logger.info(f"    Compute {n_perm} permutations")
                 self._surrogates = _ergcpac_perm(sco, amp, smooth=smooth,
                                                  n_jobs=n_jobs, n_perm=n_perm)
-                self.infer_pvalues(p=p)
+                self.infer_pvalues(p=p, mcp=mcp)
         return self.erpac
 
     def filterfit(self, sf, x_pha, x_amp=None, method='circular', smooth=None,
-                  n_perm=None, p=.05, edges=None, n_jobs=-1, verbose=None):
+                  n_perm=None, p=.05, mcp='fdr', edges=None, n_jobs=-1,
+                  verbose=None):
         """Extract phases, amplitudes and compute ERPAC.
 
         Parameters
@@ -624,16 +686,22 @@ class EventRelatedPac(_PacObj, _PacVisual):
             the same shape.
         method : {'circular', 'gc'}
             Name of the method for computing erpac. Use 'circular' for
-            reproducing [#f6]_ or 'gc' for a Gaussian-Copula based erpac.
+            reproducing :cite:`voytek2013method` or 'gc' for a Gaussian-Copula
+            based erpac.
         smooth : int | None
             Half number of time-points to use to produce a smoothing. Only
-            active with the Gaussian-Copula ('gc') method.
+            active with the Gaussian-Copula ('gc') method
+            :cite:`ince2017statistical`.
         n_perm : int | None
             Number of permutations to compute for assessing p-values for the
             gaussian-copula ('gc') method. Statistics are performed by randomly
             swapping phase trials
         p : float | 0.05
             Statistical threshold for the gaussian-copula ('gc') method
+        mcp : {'fdr', 'bonferroni'}
+            Correct the p-values for multiple comparisons. This is needed when
+            using the circular ERPAC (:cite:`voytek2013method`). Note that the
+            correction is performed using MNE-Python.
         edges : int | None
             Number of samples to discard to avoid edge effects due to filtering
 
@@ -641,13 +709,6 @@ class EventRelatedPac(_PacObj, _PacVisual):
         -------
         erpac : array_like
             The ERPAC estimation of shape (n_amp, n_pha, n_times)
-
-        References
-        ----------
-        .. [#f6] `Voytek et al, 2013 <https://www.ncbi.nlm.nih.gov/pubmed/
-           22986076>`_
-        .. [#f7] `Ince et al, 2017 <https://onlinelibrary.wiley.com/doi/full/10
-           .1002/hbm.23471>`_
         """
         x_amp = x_pha if not isinstance(x_amp, np.ndarray) else x_amp
         # extract phases and amplitudes
@@ -658,15 +719,19 @@ class EventRelatedPac(_PacObj, _PacVisual):
         amp = self.filter(sf, x_amp, ftype='amplitude', **kw)
         # compute erpac
         return self.fit(pha, amp, method=method, smooth=smooth, n_jobs=n_jobs,
-                        n_perm=n_perm, p=p, verbose=verbose)
+                        n_perm=n_perm, p=p, mcp=mcp, verbose=verbose)
 
-    def infer_pvalues(self, p=0.05):
+    def infer_pvalues(self, p=0.05, mcp='fdr'):
         """Infer p-values based on surrogate distribution.
 
         Parameters
         ----------
         p : float | 0.05
             Statistical threshold
+        mcp : {'fdr', 'bonferroni'}
+            Correct the p-values for multiple comparisons. This is needed when
+            using the circular ERPAC (:cite:`voytek2013method`). Note that the
+            correction is performed using MNE-Python.
 
         Returns
         -------
@@ -677,17 +742,22 @@ class EventRelatedPac(_PacObj, _PacVisual):
         # check that pac and surrogates has already been computed
         assert hasattr(self, 'erpac'), ("You should compute ERPAC first. Use "
                                         "the `fit` method")
-        assert hasattr(self, 'surrogates'), "No surrogates computed"
-        assert all([isinstance(k, np.ndarray) for k in (
-            self.erpac, self.surrogates)])
-        n_perm = self.surrogates.shape[0]
+        assert mcp in ['fdr', 'bonferroni']
 
-        logger.info(f"    Infer p-values at p={p}")
-        th = np.percentile(self.surrogates.reshape(n_perm, -1).max(1),
-                           100. * (1 - p), axis=0, interpolation='nearest')
-        self._pvalues = np.ones_like(self.erpac)
-        self._pvalues[self.erpac > th] = p
-        return self.pvalues
+        # correct the p-values for multiple comparisons (Voytek's only)
+        if "Voytek" in self.method:
+            logger.info(f"    Correct p-values for multiple-comparisons using "
+                        f"{mcp} correction of MNE-Python")
+            from mne.stats import fdr_correction, bonferroni_correction
+            fcn = fdr_correction if mcp is 'fdr' else bonferroni_correction
+            _, self._pvalues = fcn(self._pvalues, alpha=p)
+        else:
+            assert hasattr(self, 'surrogates'), "No surrogates computed"
+            # compute the p-values using maxstat (gcPAC)
+            self._pvalues = self._infer_pvalues(self.erpac, self.surrogates,
+                                                p=p)
+
+        return self._pvalues
 
     @property
     def erpac(self):
